package jp.android.irtest;

import java.io.IOException;
import java.util.ArrayList;

import org.microbridge.server.AbstractServerListener;
import org.microbridge.server.Server;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.DialogInterface.OnDismissListener;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.View;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.ToggleButton;

public class IRtestActivity extends Activity {

    /**
     * 待ち状態のダイアログのID
     */
    private static final int DIALOG_ID_WAITING = 0;
    /**
     * 完了メッセージのダイアログのID
     */
    private static final int DIALOG_ID_COMPLETE = 1;
    /**
     * 失敗メッセージのダイアログのID
     */
    private static final int DIALOG_ID_FAILURE = 2;
    /**
     * 再起動メッセージのダイアログのID
     */
    private static final int DIALOG_ID_RESTART = 3;

    private Handler mHandler;
    Server server = null;
    String str = "nodata";
    byte[] data_array;

    byte[] receive_array;
    String receive_str = "";
    ArrayList<Byte> data_list = new ArrayList<Byte>();
    /**
     * 学習モードならtrue、送信モードならfalse
     */
    private boolean mModeReceive = false;
    private int mWaitingChannel = -1;
    private boolean mIsWaiting = false;

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

        mHandler = new Handler();
        try {
            server = new Server(4567);
            server.start();
        } catch (IOException e) {
            Log.e("microbridge", "Unable to start TCP server", e);
            System.exit(-1);
        }
        server.addListener(new AbstractServerListener() {
            @Override
            public void onReceive(org.microbridge.server.Client client, byte[] data) {
                receive_array = data;
                String receivedData = "";
                for (int i = 0; i < data.length; i++) {
                    receive_str += "," + String.format("%d ", (short) data[i] & 0xFF);
                    receivedData += (data[i] & 0xFF) + ",";
                }
                if (mIsWaiting) {
                    if (receivedData.equals(0xFF + "," + 0x1F + ",")) {
//                    if (receivedData.equals("a" + ",")) {
                        // 成功
                        mHandler.post(new Runnable() {

                            public void run() {
                                showDialog(DIALOG_ID_COMPLETE);
                            }
                        });
                    } else if (receivedData.equals(0xFF + "," + 0x2F + ",")) {
//                    } else if (receivedData.equals(0x2F)) {
//                    } else {
                        // 失敗
                        mHandler.post(new Runnable() {

                            public void run() {
                                showDialog(DIALOG_ID_FAILURE);
                            }
                        });
                    }
                    dismissDialog(DIALOG_ID_WAITING);
                }
			};
        });

        // ToggleButtonのcheckと状態が変わった時のリスナの登録
        ToggleButton toggleButton = (ToggleButton) findViewById(R.id.toggleButton_r_s);
        toggleButton.setChecked(mModeReceive);
        toggleButton.setOnCheckedChangeListener(new OnCheckedChangeListener() {

            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                // ToggleButtonの状態を反映させる
                mModeReceive = isChecked;
            }
        });
        // USBのToggleButtonの状態が変わった時のリスナの登録
        ((ToggleButton) findViewById(R.id.toggleButton_usb))
                .setOnCheckedChangeListener(new OnCheckedChangeListener() {

                    public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                        try {
                            if (isChecked) {
                                server.send(new byte[] { (byte) 0xFF, (byte) 0xFB });
                            } else {
                                server.send(new byte[] { (byte) 0xFF, (byte) 0xFA });
                            }
                        } catch (IOException e) {
                            Log.e("microbridge", "problem sending TCP message", e);
                        }

                    }
                });
		}

    /**
     * ダイアログの生成
     */
    @Override
    protected Dialog onCreateDialog(int id, Bundle args) {
        switch (id) {
            case DIALOG_ID_COMPLETE:
                return new AlertDialog.Builder(this).setTitle("メッセージ").setMessage("成功しました")
                        .setPositiveButton("OK", null).create();
            case DIALOG_ID_FAILURE:
                return new AlertDialog.Builder(this).setTitle("メッセージ").setMessage("失敗しました")
                        .setPositiveButton("リトライ", null).setNegativeButton("閉じる", null).create();
            case DIALOG_ID_WAITING: {
                ProgressDialog dialog = new ProgressDialog(this);
                dialog.setMessage("入力待ち...");
                dialog.setCancelable(false);
                dialog.setButton(Dialog.BUTTON_NEGATIVE, "キャンセル",
                        new DialogInterface.OnClickListener() {

                            public void onClick(DialogInterface dialog, int which) {
                                // キャンセルボタンが押されたら
                                dialog.cancel();
                            }
                        });
                dialog.setOnCancelListener(new DialogInterface.OnCancelListener() {

                    public void onCancel(DialogInterface dialog) {
                        cancelReceive();
                    }
                });
                dialog.setOnDismissListener(new OnDismissListener() {

                    public void onDismiss(DialogInterface dialog) {
                        mIsWaiting = false;
                        // ProgressDialogは再利用に向いていないので消す
                        removeDialog(DIALOG_ID_WAITING);
                    }
                });
                return dialog;
            }
            case DIALOG_ID_RESTART:
                return new AlertDialog.Builder(this).setTitle("メッセージ").setMessage("再起動してください")
                        .setPositiveButton("OK", null).create();
        }
        return super.onCreateDialog(id, args);
    }

    /**
     * ダイアログの準備（ダイアログ表示前に呼ばれる）
     */
    @Override
    protected void onPrepareDialog(int id, Dialog dialog, Bundle args) {
        super.onPrepareDialog(id, dialog, args);
        switch (id) {
            case DIALOG_ID_FAILURE:
                ((AlertDialog) dialog).setButton(DialogInterface.BUTTON_POSITIVE, "リトライ",
                        new DialogInterface.OnClickListener() {

                            public void onClick(DialogInterface dialog, int which) {
                                Button_Receive(mWaitingChannel);
                            }
                        });
                break;
            case DIALOG_ID_WAITING:
                mIsWaiting = true;
                break;
        }
    }

    public void onButtonClick(View view) {
        int id = view.getId();
        if (mModeReceive) {
            // 学習モード
            switch (id) {
                case R.id.button_1:
                    Button_Receive(1);
                    break;
                case R.id.button_2:
                    Button_Receive(2);
                    break;
                case R.id.button_3:
                    Button_Receive(3);
                    break;
                case R.id.button_4:
                    Button_Receive(4);
                    break;
            }
        } else {
            // 送信モード
            switch (id) {
                case R.id.button_1:
                    Button_Send(1);
                    break;
                case R.id.button_2:
                    Button_Send(2);
                    break;
                case R.id.button_3:
                    Button_Send(3);
                    break;
                case R.id.button_4:
                    Button_Send(4);
                    break;
            }
        }
    }

    private void cancelReceive() {
        dismissDialog(DIALOG_ID_WAITING);
        // TODO キャンセル処理
        showDialog(DIALOG_ID_RESTART);
    }

    private void Button_Receive(int ch) {
        showDialog(DIALOG_ID_WAITING);
        mWaitingChannel = ch;
        
        try {
            server.send(new byte[] { (byte) 0xFF, (byte) 0xFE, (byte) ch });
        } catch (IOException e) {
            Log.e("microbridge", "problem sending TCP message", e);
        }
    }

    private void Button_Send(int ch) {
        try {
            server.send(new byte[] { (byte) 0xFF, (byte) 0xFD, (byte) ch });

        } catch (IOException e) {
            Log.e("microbridge", "problem sending TCP message", e);
        }

    }

}